﻿using LinqToDB;
using NUnit.Framework;
using System.Linq;

namespace Coscine.Database.Tests
{
    [TestFixture]
    public class ModelTests
    {
        private Migration.Migrator _migrator;
        private string _providerName;
        private string _connectionString;

        [OneTimeSetUp]
        public void Init()
        {
            var targetAssembly = typeof(ModelTests).Assembly;
            _migrator = new Migration.Migrator(targetAssembly);
            _providerName = "SqlServer.2008";
            _connectionString = _migrator.GetDbConnectionString();
        }

        [SetUpAttribute]
        public void SetUp()
        {
            _migrator.MigrateUp();
        }

        [TearDown]
        public void TearDown()
        {
            _migrator.RollBack(2);
        }

        [Test]
        public void TestRead()
        {
            using (var db = new TestCoscineDb(_providerName, _connectionString))
            {
                var q =
                    from p in db.TestUsers
                    select p;

                Assert.True(q.Any(e => e.UserId == 1));
                Assert.True(q.Count() == 3);
            }
        }

        [Test]
        public void TestInsert()
        {
            using (var db = new TestCoscineDb(_providerName, _connectionString))
            {
                db.TestProjects
                    .Value(p => p.ProjectName, "project4")
                    .Insert();
                var q2 =
                    from c in db.TestProjects
                    select c;

                Assert.True(q2.Count() == 4);
                Assert.True(q2.Any(e => e.ProjectId == 4));
            }
        }

        [Test]
        public void TestDelete()
        {
            using (var db = new TestCoscineDb(_providerName, _connectionString))
            {
                db.TestUsers
                    .Where(p => p.UserId == 3)
                    .Delete();

                var q1 =
                    from p in db.TestUsers
                    select p;

                Assert.True(q1.Count() == 2);
                Assert.False(q1.Any(e => e.UserId == 3));
            }
        }

        [Test]
        public void TestJoin()
        {
            using (var db = new TestCoscineDb(_providerName, _connectionString))
            {
                var q =
                    from u in db.TestUsers
                    from p in db.TestProjects.Where(proj => proj.ProjectId == u.UserId)
                    where u.UserId == 1
                    select new { uid = u.UserId, pname = p.ProjectName };

                Assert.True(q.All(e => e.uid == 1));
                Assert.True(q.All(e => e.pname == "project2"));
            }
        }
    }
}
